/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#ifndef INCL_WINSYNC_SOURCE_CONFIG
#define INCL_WINSYNC_SOURCE_CONFIG

#include "spds/SyncSourceConfig.h"

/*
 * --------------------- WindowsSyncSourceConfig class -----------------------------------
 * This class groups all configuration properties for a WindowsSyncSource.
 * WindowsSyncSourceConfig has an external link to SyncSourceConfig ('s'), where all common
 * properties are stored. Client-specific properties of WindowsSyncSource are members
 * of this class.
 *
 * Class members:
 * --------------
 * folderPath          : the full path of correspondent Outlook folder
 * useSubfolders       : true if the correspondent Outlook folder will be synced with
 *                       all subfolders included
 *
 * (see spds/SyncSourceConfig.h for common members)
 * 
 */
class WindowsSyncSourceConfig {

private:

    // Properties
    char* folderPath;
    bool  useSubfolders;

    // 'true' if the source has been synced (TODO: use the SOURCE_STATE)
    bool isSynced;

    // Timestamp of the last finished sync.
    long endTimestamp;


    // Pointer to (external) original SyncSourceConfig object, to retrieve
    // all common properties: we MUST get/set common properties from 
    // a unique place, so get/set methods for common properties are overrided
    // and linked to  methods of original SyncSourceConfig 's'.
    SyncSourceConfig* s;


    // Initialize all members.
    void initialize();

    // Sets the values of this object with with the values from the given
    // WindowsSyncSourceConfig object.
    void assign(const WindowsSyncSourceConfig& wsc);


public:

    //Constructs a new WindowsSyncSourceConfig object.
    // Initialize members and link the SyncSourceConfig pointer passed (mandatory not NULL).
    WindowsSyncSourceConfig(SyncSourceConfig* sc);

    // Copy constructor
    WindowsSyncSourceConfig(const WindowsSyncSourceConfig& wsc);

     //Destructor
    ~WindowsSyncSourceConfig();

    // Assign operator
    WindowsSyncSourceConfig& operator = (const WindowsSyncSourceConfig& wsc) {
        assign(wsc);
        return *this;
    } 

    // Return the pointer to external SyncSourceConfig object 
    // used for common properties.
    SyncSourceConfig* getCommonConfig();


    //
    // ----------------------------- set/get methods -----------------------------
    //
    _declspec(dllexport) const char*   getFolderPath()     const;
    _declspec(dllexport) bool          getUseSubfolders()  const;
    _declspec(dllexport) long          getEndTimestamp()   const;

    _declspec(dllexport)void setFolderPath    (const char*   v);
    _declspec(dllexport)void setUseSubfolders (bool          v);
    _declspec(dllexport)void setEndTimestamp  (long          v);

    // Common properties: get original values in SSconfig 's'
    const char*   getName()           const     { return s->getName()          ; }
    const char*   getURI()            const     { return s->getURI()           ; }
    const char*   getSyncModes()      const     { return s->getSyncModes()     ; }
    const char*   getType()           const     { return s->getType()          ; }
    const char*   getSync()           const     { return s->getSync()          ; }
    const char*   getEncoding()       const     { return s->getEncoding()      ; }
    const char*   getVersion()        const     { return s->getVersion()       ; }
    const char*   getSupportedTypes() const     { return s->getSupportedTypes(); }
    CTCap         getCtCap()          const     { return s->getCtCap()         ; }
    unsigned long getLast()           const     { return s->getLast()          ; }
    const char*   getEncryption()     const     { return s->getEncryption()    ; }

    // Common properties: set original values in SSconfig 's'
    void setName          (const char*   v)     { s->setName(v)                ; }
    void setURI           (const char*   v)     { s->setURI(v)                 ; }
    void setSyncModes     (const char*   v)     { s->setSyncModes(v)           ; }
    void setType          (const char*   v)     { s->setType(v)                ; }
    void setSync          (const char*   v)     { s->setSync(v)                ; }
    void setEncoding      (const char*   v)     { s->setEncoding(v)            ; }
    void setVersion       (const char*   v)     { s->setVersion(v)             ; }
    void setSupportedTypes(const char*   v)     { s->setSupportedTypes(v)      ; }
    void setCtCap         (CTCap         v)     { s->setCtCap(v)               ; }
    void setLast          (unsigned long v)     { s->setLast(v)                ; }
    void setEncryption    (const char*   v)     { s->setEncryption(v)          ; }


    bool getIsSynced() const;
    void setIsSynced(bool v);
};

#endif
